/**
 * MyCacheLoader.java
 */
package com.example.caching;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Map.Entry;
/**
 * MyCacheLoader.java
 */
import java.util.Set;

import org.infinispan.Cache;
import org.infinispan.container.InternalEntryFactory;
import org.infinispan.container.InternalEntryFactoryImpl;
import org.infinispan.container.entries.InternalCacheEntry;
import org.infinispan.container.versioning.EntryVersion;
import org.infinispan.loaders.AbstractCacheLoader;
import org.infinispan.loaders.CacheLoaderConfig;
import org.infinispan.loaders.CacheLoaderException;
import org.infinispan.loaders.CacheLoaderMetadata;
import org.infinispan.marshall.StreamingMarshaller;

/**
 * Custom CacheLoader
 */
@CacheLoaderMetadata(configurationClass = MyCacheLoaderConfig.class)
public class MyCacheLoader extends AbstractCacheLoader  {

	public static final String KEY_1 = "MyFirstKey";
	public static final String KEY_2 = "ABC";
	public static final String KEY_3 = "DEF";
	public static final String KEY_4 = "Tyler";
	public static final String KEY_5 = "Narrator";
	public static final String VALUE_1 = "MyFirstValue";
	public static final String VALUE_2 = "123";
	public static final String VALUE_3 = "456";
	public static final String VALUE_4 = "It's only after we've lost everything that we're free to do anything.";
	public static final String VALUE_5 = "This is your life and it's ending one minute at a time.";

	private Map<Object, Object> dataset;
	
	private String prefix;

	public MyCacheLoader() {
		this.dataset = new HashMap<Object, Object>( 5 );
		this.dataset.put( KEY_1, VALUE_1 );
		this.dataset.put( KEY_2, VALUE_2 );
		this.dataset.put( KEY_3, VALUE_3 );
		this.dataset.put( KEY_4, VALUE_4 );
		this.dataset.put( KEY_5, VALUE_5 );
	}
	
	public void init( CacheLoaderConfig config, Cache<?, ?> cache, StreamingMarshaller m ) {
		MyCacheLoaderConfig cfg = (MyCacheLoaderConfig)config;
		this.prefix = cfg.getPrefix();
	}

	/**
	 * @see org.infinispan.loaders.CacheLoader#load(java.lang.Object)
	 */
	public InternalCacheEntry load( Object key ) throws CacheLoaderException {
		InternalCacheEntry newEntry = null;

		String loadedObject = (String)this.dataset.get( key );
		if ( loadedObject != null ) {
			InternalEntryFactory factory = new InternalEntryFactoryImpl();
			newEntry = factory.create( key, this.prefix + loadedObject, (EntryVersion)null );
		}

		return newEntry;
	}

	/**
	 * @see org.infinispan.loaders.CacheLoader#loadAll()
	 */
	public Set<InternalCacheEntry> loadAll() throws CacheLoaderException {
		Set<InternalCacheEntry> newEntries = null;

		Map<Object, Object> loadedEntries = this.dataset;
		if ( loadedEntries == null || loadedEntries.size() == 0 ) {
			newEntries = new HashSet<InternalCacheEntry>( 0 );
		} else {
			newEntries = new HashSet<InternalCacheEntry>( loadedEntries.size() );
			InternalEntryFactory factory = new InternalEntryFactoryImpl();
			for ( Entry<Object, Object> entry : loadedEntries.entrySet() ) {
				newEntries.add( factory.create( entry.getKey(), this.prefix + (String)entry.getValue(), (EntryVersion)null ) );
			}
		}

		return newEntries;
	}

	/**
	 * @see org.infinispan.loaders.CacheLoader#load(int)
	 */
	public Set<InternalCacheEntry> load( int numEntries ) throws CacheLoaderException {
		throw new RuntimeException( "Not implemented" );
	}

	/**
	 * @see org.infinispan.loaders.CacheLoader#loadAllKeys(java.util.Set)
	 */
	public Set<Object> loadAllKeys( Set<Object> keysToExclude ) throws CacheLoaderException {
		throw new RuntimeException( "Not implemented" );
	}

	/**
	 * @see org.infinispan.loaders.CacheLoader#start()
	 */
	public void start() throws CacheLoaderException {
		// Nothing to do
	}

	/**
	 * @see org.infinispan.loaders.CacheLoader#stop()
	 */
	public void stop() throws CacheLoaderException {
		// Nothing to do
	}

	/**
	 * @see org.infinispan.loaders.CacheLoader#getConfigurationClass()
	 */
	public Class<? extends CacheLoaderConfig> getConfigurationClass() {
		return MyCacheLoaderConfig.class;
	}

}
