/**
 * TestCacheConfig.java
 */
package com.example.caching;

import static org.junit.Assert.assertEquals;

import java.util.Properties;

import org.infinispan.Cache;
import org.infinispan.configuration.cache.Configuration;
import org.infinispan.configuration.cache.ConfigurationBuilder;
import org.infinispan.configuration.cache.ConfigurationChildBuilder;
import org.infinispan.configuration.global.GlobalConfigurationBuilder;
import org.infinispan.configuration.global.GlobalConfigurationChildBuilder;
import org.infinispan.eviction.EvictionStrategy;
import org.infinispan.manager.DefaultCacheManager;
import org.infinispan.manager.EmbeddedCacheManager;
import org.junit.Test;

/**
 * Isolated test for cache loader configuration
 */
public class TestCacheConfig {

	private static final String TEST_PREFIX = "myownprefix_";
	private static final String PREFIX_PROPERTY = "prefix";
	private static final String TEST_CACHE_NAME = "test-cache";

	@Test
	public void test() {
		EmbeddedCacheManager cacheMgr = initCacheManager();

		Cache<String, String> cache = cacheMgr.getCache( TEST_CACHE_NAME );

		assertEquals( "Mismatch", TEST_PREFIX + MyCacheLoader.VALUE_3, cache.get( MyCacheLoader.KEY_3 ) );
	}

	private EmbeddedCacheManager initCacheManager() {
		EmbeddedCacheManager cacheMgr = null;

		GlobalConfigurationChildBuilder globalConfigBuilder = GlobalConfigurationBuilder.defaultClusteredBuilder().transport()
				.addProperty( "configurationFile", "jgroups-udp.xml" );
		globalConfigBuilder.globalJmxStatistics().enable();

		cacheMgr = new DefaultCacheManager( globalConfigBuilder.build() );

		cacheMgr.defineConfiguration( TEST_CACHE_NAME, buildCacheConfiguration() );

		return cacheMgr;
	}

	private Configuration buildCacheConfiguration() {

		int maxEntries = 100;

		ConfigurationChildBuilder configBuilder = new ConfigurationBuilder();

		// Instantiate cache loader if defined
		Properties loaderProperties = new Properties();
		loaderProperties.put( PREFIX_PROPERTY, TEST_PREFIX );

		configBuilder.loaders().addCacheLoader().cacheLoader( new MyCacheLoader() ).withProperties( loaderProperties );

		// Eviction rules setup
		configBuilder = configBuilder.eviction().strategy( EvictionStrategy.LIRS ).maxEntries( maxEntries );

		return configBuilder.build();
	}

}
